%
% Demo_BID_ADMM_UBC: Demo of the BID method proposed in [1] [2], with the possibility of assuming
% Unknown Boundary Conditions (UBC).
%                  Demo applied to a synthetic experiment of cameraman image blurred uncycliclic with a 9x9 uniform blur, at 40dB of BSNR.
%                  The stopping iteration can be selected based on an adapted ISNR measure (in the subfolder ISNR_for_BID),
%                  or on the later measures of whiteness [3,4], whose code is in the subfolder DemoWhiteness.
%
% References on BID:
% [1] M. S. C. Almeida and  M. A. T. Figueiredo,  . (JOURNAL paper)
%
% [2] M. S. C. Almeida and M. A. T. Figueiredo, "Blind Image Deblurring with Unknown Boundaries Using the
%     Alternating Direction Method of Multipliers", IEEE International Conf. on Image Processing  ICIP2013,
%     Melbourne, Australia, September, 2013. (CONFERENCE paper)
%
% References on measures of whiteness:
% [3] M. S. C. Almeida, M. A. T. Figueiredo, "New stopping criteria for iterative blind image deblurring based on
%     residual whiteness measures", IEEE Statistical Signal Processing Workshop - SSP'2011, Nice, France, 2011.
% [4] M. S. C. Almeida, M. A. T. Figueiredo, "Parameter Estimation for Blind and Non-Blind Deblurring Using Residual
%     Whiteness Measures", IEEE Trans. on Image Processing, 2013 (accepted).
%
%
% Mariana S. C. Almeida
% Instituto de Telecomunicaes, Lisbon, Portugal
% marianascalmeida@gmail.com

clear all,
close all

addpath ./images

crt_UBC = 1;    % crt_UBC = 1 - blurring with UBC (extending the image with its border pixels)
% crt_UBC = 0 - blurring assuming Cyclic Boundary Conditions

% LOAD image:
x0 = double(imread('Motbat4256_crop.tif')); % load image. 
% Other images: 'lena256.bmp' , 'cameraman.tif', 'Satalite.tif'
%               'testpat1.png', 'barbara256.bmp'
x0 = x0 - min(x0(:));  %
x0 = x0./max(x0(:));   % x0 in the range [0,1]

% DEGRADE image:
y = x0;

% method's OPTIONS: see "help_BID_NI_UBC" for full description on the method's options.
opt.filter_size = [23 23];    % opt.BlurDim = 15 - size of the estimated blurring filter (15x15, larger than the real blur of size 9x9)
opt.ctr_UBC = 1;            % opt.crt_UBC = 1 - deblurring with Unknown Boundary Conditions.
                            % opt.crt_UBC = 0 - deblurring assuming Cyclic Boundary Conditions.
opt.lambda0 = 0.5;      % opt.lambda0 = 0.5 - first values for the regularization parameter lambda.
opt.lambda_min = 1e-15; % opt.lambda_min = 1e-15 - last values for the regularization parameter lambda.
opt.ratio = 2;          % opt.ration = 1.5  - geometric ration for decreasing the regularization parameter lambda.
opt.q_vec = [0.5 0.5];  % opt.pp 0.5 - vector with the values of the spartity parameter along the iterations.
%opt.x0 = x0;            % opt.x0 - sharp image for computing the ISNR.
opt.ctr_isnr = 1;       % opt.ctr_isnr = 1 - computes the (adapted) ISNR; requires opt.x0; Stops the method automatically when the ISNR decreases 4 times.
                        % opt.ctr_isnr = 0 - does not compute ISNR.
opt.ctr_plot = 1;       % opt.ctr_plot = 1 - plots the results along iterations: image estimate, filter estimate, isnr (if computed).
                        % opt.ctr_plot = 0 - does not plot any result
opt.ctr_print = 1;      % opt.ctr_print = 1 - prints some results: iteration, time, isnr (if computed).
                       % opt.ctr_print = 0 - does not prints results.
opt.max_iter = 15;

% % % % % % % % % % % % % % % % % % % % % % %
% CALLING the BID method of [1]:
% see "help_BID_ADMM-UBC" for full discription on the method's options.
% % % % % % % % % % % % % % % % % % % % % % %
[XX HH TIME ISNRs opt] = BID_ADMM_UBC( y , opt );

figure, imagesc( max(min( y ,1),0) ), colormap gray, axis image, title('observed blurred image')
best_iter = 11;  
figure, imagesc( max(min(XX(:,:,best_iter),1),0) ), colormap gray, axis image, title(strcat('result at it=',num2str(best_iter)))
%figure, imagesc( max(min(HH(:,:,best_iter-1),1),0) ), colormap gray, axis image, title(strcat('result at it=',num2str(best_iter)))
