%
%   x = shrinkage( y , lambda ,q)
%
% solve:  min 1/2 (x-y)^2 + lambda*|x|^q  ,   min 1/2 (x-y)^2 + lambda*(x^2)^q/2
%          x                               x
%
% q = 0 (hard thresholding):  x = y.*(abs(y)> lambda)
%
% q = 1 (soft thresholding):  x = sign(y).*max(abs(y)- lambda,0)
%
% q generico: x = sign(y).*w : w>0 e w + q*lambda w^(q-1) - abs(y) = 0
%

function x_min = shrinkage( y , lambda , q, epsilon )

try q;
catch me, q=1; % default: soft-thresholding
end

try epsilon;
catch me, epsilon=0; % default: epsilon =0;
end

qq = [0 0.5 1];
switch q
    case 0, % hard-thresholding
%         x_min = y.*(abs(y) > lambda);
        x_min = y.*(abs(y.^2)/2 > lambda);
        
    case 0.5, % q=1/2
        [sz1 sz2] = size(y);
        a = -abs(y(:)); b = 1/2*lambda(:);
         zz = 2*sqrt(-repmat(a,1,3)/3).* cos(1/3.*acos( repmat( 3.*b./(2.*a).*sqrt(-3./a) ,1,3) ) - 2*pi/3*repmat([1 2 3],numel(a),1) );
        ww = zz.^2;
        ww = real(ww) + imag(ww).*(abs(imag(ww))>10^-12 );
        ww = ww.*( abs(imag(ww))==0  & ww>=0);
        ww(:,4)= 0*ones(size(ww(:,1)));
        if numel(lambda)>1
            cc = 1/2*(repmat(sign(y(:)),1,4).*ww-repmat(y(:),1,4)).^2 + repmat(lambda(:),1,4).*(abs(ww)).^q;
        else
            cc = 1/2*(repmat(sign(y(:)),1,4).*ww-repmat(y(:),1,4)).^2 + (lambda).*(abs(ww)).^q;
        end
        [min_cc indi_min] = min( cc'  );
        w = ww(:,1)'.*(indi_min==1) +  ww(:,2)'.*(indi_min==2) + ww(:,3)'.*(indi_min==3) + ww(:,4)'.*(indi_min==4);
        w = reshape(w,sz1,sz2);
        x_min = sign(y).*w;

    case 2/3,   % q=2/3=0.66(6)  - > eq. 4 Grau, mtodo em:
        % http://www.educ.fc.ul.pt/icm/icm99/icm22/frame9.htm
        
    case 1,
        x_min = sign(y).*max(abs(y)- lambda,0);
        
    otherwise
        
end


if numel(lambda)==1 & lambda==inf
    x_min = 0;
else
    %     keyboard
    %     x_min(x_min==NaN) = 0.*(lambda==inf) + x_min.*(lambda~=inf);
    x_min(find(lambda==inf)) = 0; 
end

 x_min(y==0) = 0; %.*(lambda==inf) + x_min.*(lambda~=inf);
 
end

function [f g] = myfun(x, y, lambda, q)
f = 1/2.*sum((x(:)-y(:)).^2) + lambda.*sum(abs(x(:)).^q);
g = (x(:)-y(:)) + lambda*sign(x(:)).*q.*(abs(x(:)).^(q-1));
end