% function x_min = shrinkage_CaoXu2013( y , lambda , q )
%
% solves:  min (x-y)^2 + lambda*|x|^q  ,   min (x-y)^2 + lambda*(x^2)^q/2
%           x                               x
%
% q = 0 (hard thresholding):  x = y.*(abs(y)> lambda)
%
% q = 1 (soft thresholding):  x = sign(y).*max(abs(y)- lambda,0)
%
% q=1/5 and q=2/3: uses the closed for solutions of CaoXu2013.
%
%
% Mariana S. C. Almeida; 2013
% Instituto de Telecomunicaes, Lisbon, Portugal 
% marianascalmeida@.gmail.com
%

function x_min = shrinkage_CaoXu2013( y , lambda , q )

try q;
catch me, q=1; % default: soft-thresholding
end

switch q
    case 0, % hard-thresholding
        x_min = y.*(abs(y.^2)/2 > lambda);
        
    case 0.5, % q=1/2
    
        lambda = lambda*2;
        p_lambda = (54)^(1/3)/4.*(lambda).^(2/3);
        Phi_lambda_y = acos( lambda/8 .* (abs(y)/3).^(-3/2) ); 
        
        x_min = sign(y).*2/3.*abs(y).*( 1 + cos(2*pi/3-2*Phi_lambda_y/3) );
        x_min = x_min.*(abs(y)>p_lambda);
        
    case 2/3,  
        
        lambda = lambda*2;
        
        p_lambda = 2/3*(3*lambda.^3).^(1/4);
        Pi_lambda_y = acosh( 27/16.*y.^2 .* lambda.^(-3/2) );
        A_lambda_y = 2/sqrt(3).*lambda.^(1/4).*(cosh(Pi_lambda_y/3)).^(1/2);
        A = A_lambda_y;
        
        x_min = sign(y).* ( ( A + sqrt( 2*abs(y)./A - A.^2 ) )./2 ).^3 ;
        x_min = x_min.*(abs(y)>p_lambda);
        
    case 1, % soft-thresholding
        x_min = sign(y).*max(abs(y)- lambda,0);
        
    otherwise
        
end


if numel(lambda)==1 & lambda==inf
    x_min = 0;
else
    x_min(find(lambda==inf)) = 0; 
end

x_min(y==0)=0;

end
